/* TODO_Plugin - for licensing and copyright see license.txt */

#include <StdAfx.h>

#include <CPluginCustomNode.h>

#include <Nodes/G2FlowBaseNode.h>

#include <Game.h>

#include <OculusDef.h>
#include <math.h>

namespace CustomNodePlugin
{
    class CFlowOculusNode :
        public CFlowBaseNode<eNCT_Instanced>
    {
        private:

            enum EInputPorts
            {
                EIP_Start,
                EIP_Stop,
            };

            enum EOutputPorts
            {
                EOP_Status,
                EOP_Yaw,
                EOP_Pitch,
                EOP_Roll,
            };

        public:

            HANDLE hFileMapping;
            DATAFIG* DataMem;
            int status;
            float yaw, pitch, roll;
            CFlowOculusNode( SActivationInfo* pActInfo )
            {
                int r = -1;
                hFileMapping = CreateFileMapping(  INVALID_HANDLE_VALUE,
                                                   NULL,
                                                   PAGE_READWRITE,
                                                   0,
                                                   sizeof( DATAFIG ),
                                                   "OculusDemo"  );

                DataMem = ( DATAFIG* ) MapViewOfFile(   hFileMapping,
                                                        FILE_MAP_ALL_ACCESS,
                                                        0,
                                                        0,
                                                        0 );

                if ( hFileMapping && DataMem )
                {
                    ZeroMemory( DataMem, sizeof( DATAFIG ) );
                    r = 0;
                }

                status = 0;
                yaw = pitch = roll = 0;



            }

            virtual void GetMemoryUsage( ICrySizer* s ) const
            {
                s->Add( *this );
            }

            virtual IFlowNodePtr Clone( SActivationInfo* pActInfo )
            {
                return new CFlowOculusNode( pActInfo );
            }

            virtual ~CFlowOculusNode()
            {

            }

            virtual void GetConfiguration( SFlowNodeConfig& config )
            {
                static const SInputPortConfig inputs[] =
                {
                    InputPortConfig_Void( "Start", _HELP( "Start" ) ),
                    InputPortConfig_Void( "Stop", _HELP( "Stop" ) ),
                    InputPortConfig_Null(),

                };

                static const SOutputPortConfig outputs[] =
                {
                    OutputPortConfig<int>( "Status", _HELP( "Status" ) ),
                    OutputPortConfig<float>( "Yaw", _HELP( "Yaw angle" ) ),
                    OutputPortConfig<float>( "Pitch", _HELP( "Pitch angle" ) ),
                    OutputPortConfig<float>( "Roll", _HELP( "Roll angle" ) ),
                    OutputPortConfig_Null(),
                };

                config.pInputPorts = inputs;
                config.pOutputPorts = outputs;
                config.sDescription = _HELP( "TODO" );
                config.SetCategory( EFLN_APPROVED );
            }


            void UpdateOutput( EFlowEvent evt, SActivationInfo* pActInfo )
            {
                if ( hFileMapping != NULL )
                {
                    DATAFIG* outdata = ( DATAFIG* )&DataMem->status;
                    status = outdata->status;

                    yaw = outdata->yaw;
                    pitch = outdata->pitch;
                    roll = -outdata->roll;
                }

                ActivateOutput<int>( pActInfo, EOP_Status, status );
                ActivateOutput<float>( pActInfo, EOP_Yaw, yaw );
                ActivateOutput<float>( pActInfo, EOP_Pitch, pitch );
                ActivateOutput<float>( pActInfo, EOP_Roll, roll );
            }
            virtual void ProcessEvent( EFlowEvent evt, SActivationInfo* pActInfo )
            {
                switch ( evt )
                {
                    case eFE_Suspend:
                        break;

                    case eFE_Resume:
                        break;

                    case eFE_Activate:

                        if ( IsPortActive( pActInfo, EIP_Start ) )
                        {
                            //start loop update
                            pActInfo->pGraph->SetRegularlyUpdated( pActInfo->myID, true );
                        }

                        if ( IsPortActive( pActInfo, EIP_Stop ) )
                        {
                            //stop loop update
                            ActivateOutput<int>( pActInfo, EOP_Status, 0 );
                            ActivateOutput<float>( pActInfo, EOP_Yaw, 0 );
                            ActivateOutput<float>( pActInfo, EOP_Pitch, 0 );
                            ActivateOutput<float>( pActInfo, EOP_Roll, 0 );
                            pActInfo->pGraph->SetRegularlyUpdated( pActInfo->myID, false );
                        }

                        break;

                    case eFE_SetEntityId:
                        break;

                    case eFE_Update:
                        UpdateOutput( evt, pActInfo );
                        break;
                }
            }
    };
}

REGISTER_FLOW_NODE_EX( "Custom:Oculus", CustomNodePlugin::CFlowOculusNode, CFlowOculusNode );